@@ -30,6 +30,7 @@ stopifnot(length(n) == 1, is.numeric(n), is.finite(n), n >= 1)
 print(availableCores(methods = "PBS"))
 print(availableCores(methods = "SGE"))
 print(availableCores(methods = "Slurm"))
+print(availableCores(methods = "LSF"))
 
 ## Any R options and system environment variable
 print(availableCores(methods = c("width", "FOO_BAR_ENV"),
@@ -41,6 +42,16 @@ res <- try(availableCores(methods = "FOO_BAR_ENV"), silent = TRUE)
 stopifnot(inherits(res, "try-error"))
 
 
+ncores0 <- 42L
+
+message("*** LSF ...")
+Sys.setenv(LSB_DJOB_NUMPROC = as.character(ncores0))
+ncores <- availableCores(methods = "LSF")
+print(ncores)
+stopifnot(ncores == ncores0)
+message("*** LSF ... done")
+
+
 message("*** Internal detectCores() ...")
 
 ## Option 'future.availableCores.system'
