@@ -32,6 +32,9 @@ message("*** futureAssign() - sequential w/ lazy evaluation ... DONE")
 message("*** futureAssign() - lazy = TRUE / FALSE ...")
 
 for (cores in 1:availCores) {
+  ## Speed up CRAN checks: Skip on CRAN Windows 32-bit
+  if (!fullTest && isWin32) next
+  
   message(sprintf("Testing with %d cores ...", cores))
   options(mc.cores = cores)
 
