@@ -5,6 +5,8 @@ oopts0 <- options()
 
 covr_testing <- ("covr" %in% loadedNamespaces())
 on_solaris <- grepl("^solaris", R.version$os)
+on_macos <- grepl("^darwin", R.version$os)
+on_githubactions <- as.logical(Sys.getenv("GITHUB_ACTIONS", "FALSE"))
 
 ## Default options
 oopts <- options(
@@ -78,6 +80,9 @@ requirePackages <- future:::requirePackages
 tweakExpression <- future:::tweakExpression
 whichIndex <- future:::whichIndex
 pid_exists <- future:::pid_exists
+isFALSE <- future:::isFALSE
+isNA <- future:::isNA
+supports_omp_threads <- future:::supports_omp_threads
 
 ## Local functions for test scripts
 printf <- function(...) cat(sprintf(...))
